module CommitRangeTester.JobAccessors where

import Import hiding ((</>), FilePath)

import System.IO(openFile, IOMode(ReadMode, WriteMode))

import qualified Data.Aeson as A
import qualified Data.Aeson.Types as AT
import System.Directory
import System.FilePath
import System.Environment

import Text.Read (read)
import qualified Data.List as L
import qualified Data.ByteString.Lazy as LBS


jsonStatuses :: IO A.Value
jsonStatuses = do
  queuedDir <- getEnv "QUEUED_JOB_DIR"
  queuedDirEntries <- getDirectoryContents queuedDir
  queuedJobs <- filterM (isJob queuedDir) queuedDirEntries
  queuedJobsF <- mapM (formatQueuedJob queuedDir) queuedJobs
  runningDir <- getEnv "RUNNING_JOB_DIR"
  runningDirEntries <- getDirectoryContents runningDir
  runningJobs <- filterM (isJob runningDir) runningDirEntries
  runningJobsF <- mapM (formatRunningJob runningDir) runningJobs
  completedDir <- getEnv "COMPLETED_JOB_DIR"
  completedDirEntries <- getDirectoryContents completedDir
  completedJobs <- filterM (isJob completedDir) completedDirEntries
  completedJobsF <- mapM (formatCompletedJob completedDir) completedJobs
  return $ A.toJSON $ (queuedJobsF ++ runningJobsF ++  completedJobsF :: [A.Value])

isJob :: FilePath -> FilePath -> IO Bool
isJob base dir = doesFileExist $ base </> dir </> "config"

formatQueuedJob :: FilePath -> FilePath -> IO A.Value
formatQueuedJob basedir jobID = do
  genericData <- formatJob basedir jobID
  return $ A.object $ ("status" A..= ("queued" :: String)) : genericData

formatRunningJob :: FilePath -> FilePath -> IO A.Value
formatRunningJob basedir jobID = do
  genericData <- formatJob basedir jobID
  sh <- openFile (basedir </> jobID </> "status") ReadMode
  currentCommit <- hGetLine sh
  hClose sh
  return $ A.object $
    ("status" A..= ("running" :: String)) :
    ("currentCommit" A..= (currentCommit :: String)) :
    genericData

formatCompletedJob :: FilePath -> FilePath -> IO A.Value
formatCompletedJob basedir jobID = do
  genericData <- formatJob basedir jobID
  return $ A.object $ ("status" A..= ("completed" :: String)) : genericData

formatJob :: FilePath -> FilePath -> IO [AT.Pair]
formatJob basedir jobID = do
  ch <- openFile (basedir </> jobID </> "config") ReadMode
  scriptName <- hGetLine ch
  startCommit <- hGetLine ch
  endCommit <- hGetLine ch
  hClose ch
  return [ "id" A..= jobID
         , "scriptName" A..= (scriptName :: String)
         , "startCommit" A..= (startCommit :: String)
         , "endCommit" A..= (endCommit :: String)
         ]




nextID :: IO FilePath
nextID = do
  queuedDir <- getEnv "QUEUED_JOB_DIR"
  queuedDirEntries <- getDirectoryContents queuedDir
  queuedJobs <- filterM (isJob queuedDir) queuedDirEntries
  let lastQueuedJob = L.last $ L.sort queuedJobs
  return $ show $ (read lastQueuedJob :: Int) + 1

createJob :: Text -> Text -> FilePath -> LBS.ByteString -> IO ()
createJob startCommit endCommit scriptFileName script = do
  jobID <- nextID
  queuedDir <- getEnv "QUEUED_JOB_DIR"
  createDirectory (queuedDir </> jobID)
  sh <- openFile (queuedDir </> jobID </> scriptFileName) WriteMode
  LBS.hPut sh script
  hClose sh
  ch <- openFile (queuedDir </> jobID </> "config") WriteMode
  hPutStrLn ch scriptFileName
  hPutStrLn ch startCommit
  hPutStrLn ch endCommit
  hClose ch
