module Handler.Jobs where

import Import
import Text.Hamlet (hamletFile)
import Text.Julius (juliusFile)
import Text.Cassius (cassiusFile)

import Data.Maybe as M
import Data.Conduit.List (consume)
import qualified Data.ByteString.Lazy as LBS

import CommitRangeTester.JobAccessors

getJobsListR :: Handler TypedContent
getJobsListR = selectRep $ do
  provideRep $ defaultLayout $ do
    setTitle "Jobs list"
    toWidgetHead $(hamletFile "templates/dojoInclude.hamlet")
    $(whamletFile "templates/jobList.hamlet")
    toWidget $(juliusFile "templates/jobList.julius")
    toWidget $(cassiusFile "templates/jobList.cassius")
  provideRep $ liftIO jsonStatuses

jobStatusGrid :: Widget
jobStatusGrid = do
  -- The grid markup
  [whamlet|<div #status-grid>|]
  -- The JS to make it work
  toWidget $(juliusFile "templates/statusGrid.julius")
  -- The basic styling
  toWidget $(cassiusFile "templates/statusGrid.cassius")

-- Eventually used for adding a new job
postJobsListR :: Handler Html
postJobsListR = defaultLayout $ do
  startCommit <- lookupPostParam "start-commit"
  endCommit <- lookupPostParam "end-commit"
  scriptFile <- lookupFile "script"
  script <- LBS.fromChunks <$> (fileSource (M.fromJust scriptFile) $$ consume)
  liftIO $ createJob (M.fromJust startCommit) (M.fromJust endCommit)
    (unpack $ fileName (M.fromJust scriptFile)) script
  redirect JobsListR

getNewJobR :: Handler Html
getNewJobR = defaultLayout $ do
  setTitle "Create job"
  $(whamletFile "templates/newJob.hamlet")

nav :: Widget
nav = do
  [whamlet|<div .nav>
   <a .return href=@{JobsListR}>Return to status dashboard</a>
    |]
  toWidget $(cassiusFile "templates/nav.cassius")
